import { Controller, UsePipes } from '@nestjs/common';
import { QueryBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { GRPC_SERVICE_NAME } from '@src/app.constants';
import { ConfigurationMapper } from '@modules/configuration/configuration.mapper';
import { GetConfigurationRequestDto } from './dtos/get-configuration.request.dto';
import { ConfigurationResponseDto } from '../dtos/configuration.response.dto';
import { GetConfigurationQuery } from '@modules/configuration/core/application/queries/get-configuration/get-configuration.query';
import {
  NotFoundException,
  RpcExceptionCode,
  RpcValidationPipe,
} from '@mobicoop/ddd-library';
import { Value } from '@mobicoop/configuration-module';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class GetConfigurationGrpcController {
  constructor(
    protected readonly mapper: ConfigurationMapper,
    private readonly queryBus: QueryBus,
  ) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'Get')
  async get(
    data: GetConfigurationRequestDto,
  ): Promise<ConfigurationResponseDto> {
    try {
      const value: Value = await this.queryBus.execute(
        new GetConfigurationQuery(data.domain, data.key),
      );
      return this.mapper.toResponse(data, value);
    } catch (e) {
      if (e instanceof NotFoundException) {
        throw new RpcException({
          code: RpcExceptionCode.NOT_FOUND,
          message: e.message,
        });
      }
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: e.message,
      });
    }
  }
}
