import { Test, TestingModule } from '@nestjs/testing';
import { SetConfigurationRequestDto } from '@modules/configuration/interface/grpc-controllers/dtos/set-configuration.request.dto';
import { SetConfigurationService } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.service';
import { CONFIGURATION_REPOSITORY } from '@modules/configuration/configuration.di-tokens';
import { SetConfigurationCommand } from '@modules/configuration/core/application/commands/set-configuration/set-configuration.command';
import { Domain, Type } from '@mobicoop/configuration-module';
import { ConfigurationsManagerService } from '@modules/configuration/core/application/services/configurations-manager.service';

const setConfigurationRequest: SetConfigurationRequestDto = {
  domain: Domain.CARPOOL,
  key: 'seatsProposed',
  value: '3',
};

const mockConfigurationRepository = {
  set: jest
    .fn()
    .mockImplementationOnce(() => ({}))
    .mockImplementationOnce(() => {
      throw new Error();
    }),
};

const mockConfigurationsManagerService = {
  identifierType: jest.fn().mockImplementation(() => Type.INT),
  cast: jest.fn().mockImplementation(() => 3),
};

describe('Set Configuration Service', () => {
  let setConfigurationService: SetConfigurationService;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: CONFIGURATION_REPOSITORY,
          useValue: mockConfigurationRepository,
        },
        {
          provide: ConfigurationsManagerService,
          useValue: mockConfigurationsManagerService,
        },
        SetConfigurationService,
      ],
    }).compile();

    setConfigurationService = module.get<SetConfigurationService>(
      SetConfigurationService,
    );
  });

  it('should be defined', () => {
    expect(setConfigurationService).toBeDefined();
  });

  describe('execution', () => {
    const setConfigurationCommand = new SetConfigurationCommand({
      identifier: {
        domain: setConfigurationRequest.domain,
        key: setConfigurationRequest.key,
      },
      value: setConfigurationRequest.value,
    });
    it('should set an existing configuration item', async () => {
      jest.spyOn(mockConfigurationRepository, 'set');
      await setConfigurationService.execute(setConfigurationCommand);
      expect(mockConfigurationRepository.set).toHaveBeenCalledTimes(1);
    });
    it('should throw an error if something bad happens on configuration item update', async () => {
      await expect(
        setConfigurationService.execute(setConfigurationCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
